@@ -1,9 +1,9 @@
-# import pandas as pd
-# import numpy as np
-# from pgmpy.inference import VariableElimination
-# from pgmpy.models import BayesianNetwork, NaiveBayes
-# from pgmpy.estimators import ExhaustiveSearch, HillClimbSearch, TreeSearch
-# from pgmpy.factors.discrete import TabularCPD
+import pandas as pd
+import numpy as np
+from pgmpy.inference import VariableElimination
+from pgmpy.models import BayesianNetwork, NaiveBayes
+from pgmpy.estimators import ExhaustiveSearch, HillClimbSearch, TreeSearch
+from pgmpy.factors.discrete import TabularCPD
 
 # %%
 import bnlearn as bn
@@ -12,6 +12,20 @@ import bnlearn as bn
 # print(dir(bn.parameter_learning))
 # print(dir(bn.inference))
 
+# %% Naive Bayesian model
+df = bn.import_example('random')
+# Structure learning
+model = bn.structure_learning.fit(df, methodtype='nb', root_node="B", verbose=4, n_jobs=1)
+model = bn.structure_learning.fit(df, methodtype='hc', verbose=4, n_jobs=1)
+model = bn.structure_learning.fit(df, methodtype='cs', verbose=4, n_jobs=1)
+model = bn.structure_learning.fit(df, methodtype='cl', verbose=4, n_jobs=1)
+model = bn.structure_learning.fit(df, methodtype='tan', class_node="A", verbose=4, n_jobs=1)
+model = bn.structure_learning.fit(df, methodtype='ex', verbose=4, n_jobs=1)
+model = bn.independence_test(model, df, prune=True)
+# Plot
+bn.plot(model)
+
+
 # %%
 # from pgmpy.estimators import BayesianEstimator
 # from pgmpy.utils import get_example_model
@@ -24,11 +38,12 @@ import bnlearn as bn
 # new_model.add_cpds(*cpds)
  
 # %% LOAD BIF FILE
+import bnlearn as bn
 DAG = bn.import_DAG('water', verbose=0)
 # Sampling
 df = bn.sampling(DAG, n=1000)
 # Parameter learning
-model = bn.parameter_learning.fit(DAG, df, smooth=None)
+model = bn.parameter_learning.fit(DAG, df, scoretype='bdeu', smooth=None)
 
 # %%
 import bnlearn as bn
@@ -339,7 +354,7 @@ print(cpd_C)
 cpd_D = TabularCPD(variable='D', variable_card=2, values=[[0.4, 0.9], [0.6, 0.1]], evidence=['A'], evidence_card=[2])
 print(cpd_D)
 
-DAG = bn.make_DAG(DAG, CPD=[cpd_A, cpd_B, cpd_C, cpd_D], checkmodel=True)
+DAG = bn.make_DAG(DAG, CPD=[cpd_A, cpd_B, cpd_C, cpd_D], checkmodel=False)
 bn.print_CPD(DAG, checkmodel=True)
 
 # %% Adjust some edge properties
@@ -564,7 +579,6 @@ model = bn.structure_learning.fit(df)
 model = bn.structure_learning.fit(df, methodtype='hc')
 
 # %% Predict
-
 df = bn.import_example('asia')
 edges = [('smoke', 'lung'),
          ('smoke', 'bronc'),
